import sys

def isString(data):
    if (sys.version_info[0] == 3):
        return isinstance(data, str)
    return isinstance(data, basestring)


def isInteger(data):
    if (sys.version_info[0] == 3):
        return isinstance(data, int)
    return isinstance(data, (int, long))



import os, sys, time, threading, traceback, signal, subprocess, psutil

from .actorlib import send, receive

from ..eval.Interpreter import Interpreter

from ..visitor.Parser import parseFromString

from ..actors.console import Console

debug = False

def infanticide(pid):
    try:
      parent = psutil.Process(pid)
    except psutil.NoSuchProcess:
      return
    children = parent.children(recursive=True)
    if debug:
        sys.stderr.write('The children of {0} are {1}\n'.format(pid, children))
    for p in children:
        os.kill(p.pid, signal.SIGKILL)
        if debug:
            sys.stderr.write('Sent signal {1} to {0}\n'.format(p.pid, signal.SIGKILL))


def handler(signum, frame):
    if debug:
        sys.stderr.write('Signal handler called with signal {0}\n'.format(signum))
        sys.stderr.flush()
    infanticide(os.getpid())
    sys.exit()


def main():
    launch(sys.argv[1] if len(sys.argv) == 2 else "noname")
    return 0


class Main(object):

    retries = 1

    launchConsole = False

    # a singleton instance
    myself = None
    
    def __init__(self, name):
        """Creates an plambda Actor object with the given name.
        """
        self.name = name
        self.interpreter = Interpreter()
        signal.signal(signal.SIGINT, handler)
        if Main.myself is None:
            Main.myself = self
        else:
            raise Exception("plambda.actor.pyactor.Main should have a singleton instance!")



        
    def run(self):
        """The main thread ever ready to launch the console.
        """
        fails = 0

        self.oracle = threading.Thread(target=oracle, args=(self, ))
        self.oracle.daemon = True
        self.oracle.start()
        
        while True:
            time.sleep(1)
            if Main.launchConsole:
                Main.launchConsole = False
                self.console = Console(self.interpreter)
                self.console.mainloop()

                
        return 0
    
def notify(message):
    if debug:
        sys.stderr.write('{0}\n'.format(message))
        sys.stderr.flush()


def oracle(actor):
    """The Read Eval Message Loop.
    """
    fails = 0

    while True:
        incoming = receive()
        if incoming is None:
            fails += 1
            if fails > Main.retries:
                return 0
            else:
                continue
        (sender, msg) = incoming
        thread = threading.Thread(target=eval, args=(actor.interpreter, msg))
        #thread needs to be a daemon so that the actor itself can die in peace.
        thread.daemon = True
        thread.start()
            

        
def eval(interpreter, message):
    """Evaluates the message in the given interpreter.
    """
    try:
        val = interpreter.evaluateString(message)
        notify('eval: {0} evaluated to {1}'.format(message, val))
    except Exception as e:
        sys.stderr.write('plambda.actors.pyactor.Main exception: {0}\n'.format(e))
        if debug:
            traceback.print_exc(file=sys.stderr)


def launch(name):
    main = Main(name)
    main.run()
    return 0
    



"""
Freeping Creaturism:

This is the all important version number used by pip.


Version history:

1.0.0   7/1/2016  Paging in the pip process.

1.0.1   8/5/2016  pyactor shutsdown (i.e. kills) any children (like SITL etc).

1.0.2   8/18/2016  kwapply added.

1.0.3   8/18/2016  modify added.

1.0.4   8/21/2016  single quote strings added. isint isfloat added. isobject just (not (isnone))

1.0.5   8/23/2016  The read-eval-print loop gets a makeover. handles multiline paste and prints definitions.

1.0.6   8/25/2016  PLambda gets a console.

1.0.7   8/27/2016  Updated all relative imports to follow https://www.python.org/dev/peps/pep-0328/

1.0.8   9/02/2016  CPS (Continuation Passing Stype) installed as the default. Can switch to recursive via a mode in plambda.eval.Interpreter

1.0.9   9/10/2016  Various bug fixes in the CPS interpreter.

1.0.10  9/27/2016  Fixed the lookup mechanism in the CPS interpreter.

"""

plambda_version='1.0.10'

